﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Drawing2D;
using System.Threading;
using System.Drawing.Imaging;

namespace datasafe
{
    public partial class Form1 : Form
    {
  
        int _totalFiles = 0;

        public Form1()
        {
            InitializeComponent();

            ImageList imageList = new ImageList();

            imageList.Images.Add(datasafe.Properties.Resources.padlock_icon);

            imageList.Images.Add(datasafe.Properties.Resources.blank_icon);

            listView2.KeyUp += new KeyEventHandler(listBox1_KeyUp);

            listView2.SmallImageList = imageList;

            listView2.View = View.Details;

            listView2.DragDrop +=new DragEventHandler(listView2_DragDrop);
            listView2.DragEnter +=new DragEventHandler(listView2_DragEnter);

        
            label_status.Text = "";

            this.FormClosing += new FormClosingEventHandler(Form1_FormClosing);


        }

        void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            _stop = true;
        }

        private static Image resizeImage(Image imgToResize, Size size)
        {
            int sourceWidth = imgToResize.Width;
            int sourceHeight = imgToResize.Height;

            float nPercent = 0;
            float nPercentW = 0;
            float nPercentH = 0;

            nPercentW = ((float)size.Width / (float)sourceWidth);
            nPercentH = ((float)size.Height / (float)sourceHeight);

            if (nPercentH < nPercentW)
                nPercent = nPercentH;
            else
                nPercent = nPercentW;

            int destWidth = (int)(sourceWidth * nPercent);
            int destHeight = (int)(sourceHeight * nPercent);

            Bitmap b = new Bitmap(destWidth, destHeight);
            Graphics g = Graphics.FromImage((Image)b);
            g.InterpolationMode = InterpolationMode.HighQualityBicubic;

            g.DrawImage(imgToResize, 0, 0, destWidth, destHeight);
            g.Dispose();

            return (Image)b;
        }
     
        void Form1_Load(object sender, System.EventArgs e)
        {
           // throw new System.NotImplementedException();
        }


        void listBox1_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                ListView.SelectedIndexCollection list = listView2.SelectedIndices;

                for (int i = list.Count-1; i >= 0; i--)
                {
                    listView2.Items.RemoveAt(i);
                }
            }
        }

        Thread workerThread;
        bool _stop = false;
        Queue<string> _fileList;

        void workerLoop()
        {
            while ( _fileList.Count > 0 )
            {
                if (_stop) break;
               
                try
                {
                    string file = _fileList.Dequeue();

                    int asInt = Convert.ToInt32(textBox1.Text);
                    float percentageShrink = (float)asInt;
                    percentageShrink /= (float)100.0;

                    Bitmap image = new Bitmap(file);

                    Size startingSize = image.Size;

                    Size endingSize = new Size();

                    endingSize.Width = (int)((float)startingSize.Width * percentageShrink);

                    endingSize.Height = (int)((float)startingSize.Height * percentageShrink);

                    string s = "file(" + file + ") ending size will be: (" + endingSize.Width.ToString() +" x " + endingSize.Height.ToString() + ")";
                    this.BeginInvoke((MethodInvoker)delegate { this.postStatusString(s); });

                    Bitmap b = new Bitmap(endingSize.Width, endingSize.Height);
                    Graphics g = Graphics.FromImage((Image)b);
                    g.InterpolationMode = InterpolationMode.HighQualityBicubic;

                    g.DrawImage(image, 0, 0, endingSize.Width, endingSize.Height);
                    g.Dispose();


                    image.Dispose();
                    image = null;
                    File.Delete(file);

                    
                    EncoderParameter qualityParam = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 90L);
                    
                    ImageCodecInfo jpegCodec = GetEncoderInfo("image/jpeg");

                    EncoderParameters encoderParams = new EncoderParameters(1);
                    encoderParams.Param[0] = qualityParam;

                    b.Save(file, jpegCodec, encoderParams);
                }
                catch (Exception ex)
                {
                    if ( _stop ) return;

                    string s = "Exception: " + ex.Message;
                    this.BeginInvoke((MethodInvoker)delegate { this.postStatusString(s); });
                }

                this.BeginInvoke((MethodInvoker)delegate { this.incrementDoneCount(); });

            }
        }

        private static ImageCodecInfo GetEncoderInfo(string mimeType)
        {
            // Get image codecs for all image formats
            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();

            // Find the correct image codec
            for (int i = 0; i < codecs.Length; i++)
                if (codecs[i].MimeType == mimeType)
                    return codecs[i];
            return null;
        } 

        void postStatusString(string s)
        {
            label_status.Text = s;
        }


        int _doneCount = 0;
        void incrementDoneCount( )
        {
            progressBar1.Value = ++ _doneCount;
        }

        void button_StartShrink_Click(object sender, System.EventArgs e)
        {
            int percentage = Convert.ToInt32(textBox1.Text);
            if (percentage < 1 || percentage > 99)
            {
                label_status.Text = "shrink percentage must be > 0 and < 100 ";
                return;
            }

            _doneCount = 0;

            _fileList = new Queue<string>();

            for (int i = listView2.Items.Count - 1; i >= 0; i--)
            {
                _fileList.Enqueue ( listView2.Items[i].Text);
            }
            workerThread = new Thread(workerLoop);
            workerThread.Start();
        }

      


        void addFileOrDirectory(string file)
        {
            if (Directory.Exists(file))
            {
                string[] directories = Directory.GetDirectories(file);

                foreach (string dir in directories)
                {
                    addFileOrDirectory(dir);
                }

                string[] files = Directory.GetFiles(file);

                foreach (string f in files)
                {
                    addFileOrDirectory(f);
                }
            }
            else if (File.Exists(file))
            {
                ListViewItem item = new ListViewItem(file);

                if (!isFileInList(file))
                {
                    FileInfo fi = new FileInfo(file);
                    if ( fi.Extension.ToLower() == ".jpg" || fi.Extension.ToLower() == ".jpeg")
                        listView2.Items.Add(item);
                }
            }
        }

        private bool isFileInList(string testFile)
        {
            for (int i = listView2.Items.Count - 1; i >= 0; i--)
            {
                string file = listView2.Items[i].Text;
                if (testFile == file)
                    return true;
            }

            return false;
        }

        private void listView2_DragEnter(object sender, DragEventArgs e)
        {
            
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true)
            {
                e.Effect = DragDropEffects.All;
            }
        }


        private void listView2_DragDrop(object sender, DragEventArgs e)
        {
            label_status.Text = "adding files...";

            try
            {
                if (listView2.Columns.Count == 0)
                {
                    listView2.Columns.Add("file");
                    listView2.Columns[0].Width = listView2.Width - 2;
                }
               
               
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);

                // loop through the string array, adding each filename to the ListBox
                foreach (string file in files)
                {
                    // is it a file or a directory?
                    addFileOrDirectory(file);
                }
            }
            catch (Exception ex)
            {
                label_status.Text = ex.Message;
            }

            _totalFiles = listView2.Items.Count;

            label_status.Text = "Completed adding files, total files : " + _totalFiles.ToString();

            progressBar1.Maximum = 0;

            progressBar1.Maximum = _totalFiles;

        }

        private void progressBar1_Click(object sender, EventArgs e)
        {

        }

       

    }



}
